import { toComputeResponse, toReqRes } from '@fastly/http-compute-js';
import { RUN_DIR } from '../constants.js';
import { setCacheControlHeaders, setCacheTagsHeaders, setVaryHeaders } from '../headers.js';
let nextHandler, nextConfig;
export default async (request) => {
    if (!nextHandler) {
        // set the server config
        const { getRunConfig, setRunConfig } = await import('../config.js');
        nextConfig = await getRunConfig();
        setRunConfig(nextConfig);
        const { getMockedRequestHandlers } = await import('./next.cjs');
        [nextHandler] = await getMockedRequestHandlers({
            port: 3000,
            hostname: 'localhost',
            dir: RUN_DIR,
            isDev: false,
        });
    }
    const { req, res } = toReqRes(request);
    res.prependListener('_headersSent', (event) => {
        const headers = new Headers(event.headers);
        setCacheControlHeaders(headers);
        setCacheTagsHeaders(headers);
        setVaryHeaders(headers, request, nextConfig);
        event.headers = Object.fromEntries(headers.entries());
        // console.log('Modified response headers:', JSON.stringify(event.headers, null, 2))
    });
    try {
        console.log('Next server request:', req.url);
        await nextHandler(req, res);
    }
    catch (error) {
        console.error(error);
        res.statusCode = 500;
        res.end('Internal Server Error');
    }
    // log the response from Next.js
    const response = { headers: res.getHeaders(), statusCode: res.statusCode };
    // console.log('Next server response:', JSON.stringify(response, null, 2))
    return toComputeResponse(res);
};
